using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Imaging;

namespace CodeJotter {
    public partial class CodeJotter : UserControl {

        private Point GetCursorPosFromPoint(Point p) {
            int XPos = (int)((p.X - leftmostGutter - fontWidth / 2 - 1) / fontWidth) + HScroller.Value;
            int YPos = (int)(p.Y / this.fontHeight);
            
            LinkedListNode<Line> FindXList = topmostLineNode;
            for (int i = 0; i < YPos; ++i) {
                if (FindXList != null && FindXList.Next != null) FindXList = FindXList.Next;
            }

            if (FindXList != null) {
                int rx = 0;
                foreach (RenderedCharacter i in FindXList.Value.CachedChars) {
                    if (i.XPosition >= XPos) break;
                    ++rx;                    
                }
                return new Point(Math.Min(rx, FindXList.Value.Characters.Count - 1), YPos + topmostLine);
            } else {
                return new Point(XPos, YPos+ topmostLine);
            }
        }

        protected override void OnMouseDown(MouseEventArgs e) {
            bool InGutter = e.X < leftmostGutter && e.X > 16;
            bool InBreakpoint = e.X < 16;

            
            CursorPos = GetCursorPosFromPoint(new Point(e.X, e.Y));
            UpdatePointers();
            CursorOn = true;
            JumpToCursor();

            if (InBreakpoint) {
                SelectionCursorPos = CursorPos;
                currentLine.Value.IsBreakpoint ^= true;
                currentLine.Value.NeedsHighlighting = true;
                Invalidate();
            } else {
                if (!ShiftPressed) {
                    SelectionCursorPos = CursorPos;
                    if (InGutter) {
                        SelectionCursorPos = new Point(0, SelectionCursorPos.Y);
                        UpdatePointers();
                        CursorPos = new Point(0, CursorPos.Y + 1);
                    }
                    UpdatePointers();
                }               
                InvalidateAll();
            }
            base.OnMouseDown(e);
        }


        
        protected override void OnMouseMove(MouseEventArgs e) {


            bool InGutter = e.X < leftmostGutter && e.X > 16;
            bool InBreakpoint = e.X < 16;


            this.Cursor = !InGutter ? (!InBreakpoint ? System.Windows.Forms.Cursors.IBeam : System.Windows.Forms.Cursors.Hand) : System.Windows.Forms.Cursors.UpArrow;


            if (!InBreakpoint) {

                if ((e.Button & MouseButtons.Left) == MouseButtons.Left) {

                    int OldY = CursorPos.Y;

                    if (InGutter) {
                        CursorX = 0;
                        Point p = GetCursorPosFromPoint(new Point(e.X, e.Y));
                        if (CursorPos.Y <= SelectionCursorPos.Y) {
                            CursorY = p.Y;
                            selectionCursorPos.X = selectedLine.Value.Characters.Count - 1;
                        } else {
                            if (p.Y == lines.Count - 1) {
                                CursorY = p.Y;
                                CursorX = currentLine.Value.Characters.Count - 1;
                            } else {
                                CursorY = p.Y + 1;
                            }
                            selectionCursorPos.X = 0;
                        }
                    } else {
                        this.CursorPos = GetCursorPosFromPoint(new Point(e.X, e.Y));
                    }
                    JumpToCursor();
                    Rectangle rc = this.ClientRectangle;

                    int Top = Math.Min(OldY, CursorY) - 1;
                    int Btm = Math.Max(OldY, CursorY) + 1;

                    rc.Height = this.FontHeight * (Btm - Top);
                    rc.Y = (Top - topmostLine) * this.FontHeight;
                    this.Invalidate(rc);
                }
            }
        }



    }
}